<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       http://www.visual4.de/
 * @since      1.0.0
 *
 * @package    v4_wp1crm_Extensions
 * @subpackage v4_wp1crm_Extensions/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    v4_wp1crm_Extensions
 * @subpackage v4_wp1crm_Extensions/public
 * @author     visual4 GmbH <info@visual4.de>
 */
class v4_wp1crm_Extensions_Public
{

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $plugin_name The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $version The current version of this plugin.
     */
    private $version;

    /**
     * The plugin helper.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $version The current version of this plugin.
     */
    private $helper;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string $plugin_name The name of the plugin.
     * @param      string $version The version of this plugin.
     */
    public function __construct($plugin_name, $version)
    {

        $this->plugin_name = $plugin_name;
        $this->version = $version;

        $this->helper = new v4_wp1crm_Extensions_Public_Helper();

        add_filter('woocommerce_form_field_info', [$this, 'woocommerce_form_field_info'], 20, 4);
        add_action('woocommerce_checkout_before_customer_details', [$this, 'order_for_customer'], 20);
        add_action('woocommerce_checkout_billing', [$this, 'order_for_customer_billing'], 5);
        add_action('woocommerce_checkout_shipping', [$this, 'order_for_customer_shipping'], 5);
        add_action('woocommerce_checkout_update_order_meta', [$this, 'order_for_customer_field_update']);
    }

    /**
     * Register the stylesheets for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueue_styles()
    {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in v4_wp1crm_Extensions_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The v4_wp1crm_Extensions_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'css/v4-wp1crm-extensions-public.css', array(), $this->version, 'all');
    }

    /**
     * Register the JavaScript for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts()
    {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in v4_wp1crm_Extensions_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The v4_wp1crm_Extensions_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        wp_enqueue_script($this->plugin_name, plugin_dir_url(__FILE__) . 'js/v4-wp1crm-extensions-public.js', array('jquery'), $this->version, true);

        $localize_script_options = [
            'required' => '<abbr class="required" title="' . esc_attr__('required', 'woocommerce') . '">*</abbr>',
            'optional' => '<span class="optional">(' . esc_html__('optional', 'woocommerce') . ')</span>',
            'attend_myself' => esc_html__('Make this ticket out to me', 'v4-wp1crm-extensions'),
            'is_user_logged_in' => is_user_logged_in(),
            'current_user' => [
                'email' => '',
                'first_name' => '',
                'last_name' => '',
                'salutation' => '',
                'company' => '',
                'phone' => '',
                'primary_address_street' => '',
                'primary_address_city' => '',
                'primary_address_postalcode' => '',
                'primary_address_country' => '',
            ]
        ];
        $user = wp_get_current_user();
        if ($user) {
            $localize_script_options['current_user']['email'] = $user->user_email;
            $localize_script_options['current_user']['first_name'] = $user->user_firstname;
            $localize_script_options['current_user']['last_name'] = $user->user_lastname;
            if (function_exists('wc_gzd_get_customer_title_options')) {
                $billing_title = get_user_meta($user->ID, 'billing_title', true);
                $wc_gzd_get_customer_title_options = wc_gzd_get_customer_title_options();
                $localize_script_options['current_user']['salutation'] = !empty($wc_gzd_get_customer_title_options[$billing_title]) ? $wc_gzd_get_customer_title_options[$billing_title] : $billing_title;
            }
            $localize_script_options['current_user']['primary_address_street'] = get_user_meta($user->ID, 'billing_address_1', true);
            $localize_script_options['current_user']['primary_address_city'] = get_user_meta($user->ID, 'billing_city', true);
            $localize_script_options['current_user']['primary_address_postalcode'] = get_user_meta($user->ID, 'billing_postcode', true);
            $localize_script_options['current_user']['company'] = get_user_meta($user->ID, 'billing_company', true);
            $localize_script_options['current_user']['phone'] = get_user_meta($user->ID, 'billing_phone', true);

            /** @var V4_Wp_Webhook_Admin $V4_Wp_Webhook_Admin */
            global $V4_Wp_Webhook_Admin;
            $billing_country = get_user_meta($user->ID, 'billing_country', true);
            $countries = $V4_Wp_Webhook_Admin->get_countries();
            $localize_script_options['current_user']['primary_address_country'] = $billing_country ? $countries[$billing_country] : '';
        }

        $localize_script_options = apply_filters('v4-wp1crm-extensions/localize-script-options', $localize_script_options);
        wp_localize_script($this->plugin_name, 'v4_wp1crm_extensions', $localize_script_options);
    }

    /**
     * Display CustomFields in Checkout
     *
     * @param $fields
     * @return mixed
     */
    public function display_custom_fields_in_checkout($fields)
    {
        $billingcustomfields = $this->helper->getAllFieldsFromOptions(get_option(v4_wp1crm_Extensions_Public_Setting::$SECTIONS['billing']['htmlid']));
        $htmlbillingfields = $this->helper->createHtmlFormFields($billingcustomfields);
        $fields = $this->helper->addFieldsToSectionAfter($fields, 'billing', $htmlbillingfields);

        $shippingcustomfields = $this->helper->getAllFieldsFromOptions(get_option(v4_wp1crm_Extensions_Public_Setting::$SECTIONS['shipping']['htmlid']));
        $htmlshippingfields = $this->helper->createHtmlFormFields($shippingcustomfields);
        $fields = $this->helper->addFieldsToSectionAfter($fields, 'shipping', $htmlshippingfields);

        $accountcustomfields = $this->helper->getAllFieldsFromOptions(get_option(v4_wp1crm_Extensions_Public_Setting::$SECTIONS['account']['htmlid']));
        $htmlaccountfields = $this->helper->createHtmlFormFields($accountcustomfields);
        $fields = $this->helper->addFieldsToSectionAfter($fields, 'account', $htmlaccountfields);

        $ordercustomfields = $this->helper->getAllFieldsFromOptions(get_option(v4_wp1crm_Extensions_Public_Setting::$SECTIONS['order']['htmlid']));
        $htmlorderfields = $this->helper->createHtmlFormFields($ordercustomfields);
        $fields = $this->helper->addFieldsToSectionAfter($fields, 'order', $htmlorderfields);

        //Add Salutation -> use "Germanized" title instead
//        $htmlsalutation = $this->helper->createSalutationHtml();
//        $fields = $this->helper->addFieldsToSectionBefore($fields, 'billing', $htmlsalutation);

        //Add USt-ID
        $wp1crm = get_option('wp1crm');
        if (isset($wp1crm['b2b']) && $wp1crm['b2b']) {
            $htmlustid = $this->helper->createUStIDHtml();
            $fields = $this->helper->addFieldsToSectionAfter($fields, 'billing', $htmlustid);
        }

        return $fields;
    }

    /**
     * Add all FieldValues to CRM Webhook
     *
     * @param $http_args
     * @return mixed
     */
    public function add_http_args_to_woocommerce_webhook($http_args)
    {
        $body = json_decode($http_args['body']);

        if (isset($body->order)) {
            $body = $this->helper->addProductUrlToWebhookBody($body);
            $body = $this->helper->addCustomFieldsToWebhookBody($body);
            $body = $this->helper->addPayPalMetaToWebhookBody($body);

            $wp1crm = get_option('wp1crm');
            if ($wp1crm['b2b']) {
                $body->order->customer->tax_information = get_post_meta(get_current_user_id(), 'tax_information');
                $body->order->tax_information = get_post_meta($body->order->id, 'tax_information');
            }

            $http_args['body'] = json_encode($body);
        }

        return $http_args;
    }

    /**
     * Add Meta Fields to Order
     *
     * @param $order_id
     */
    public function custom_order_field_update($order_id)
    {
        $billingcustomfields = $this->helper->getAllFieldsFromOptions(get_option(v4_wp1crm_Extensions_Public_Setting::$SECTIONS['billing']['htmlid']));
        $shippingcustomfields = $this->helper->getAllFieldsFromOptions(get_option(v4_wp1crm_Extensions_Public_Setting::$SECTIONS['shipping']['htmlid']));
        $accountcustomfields = $this->helper->getAllFieldsFromOptions(get_option(v4_wp1crm_Extensions_Public_Setting::$SECTIONS['account']['htmlid']));
        $ordercustomfields = $this->helper->getAllFieldsFromOptions(get_option(v4_wp1crm_Extensions_Public_Setting::$SECTIONS['order']['htmlid']));
        $customfields = array_merge($billingcustomfields, $shippingcustomfields, $accountcustomfields, $ordercustomfields);

        //Iterate and create Formfield
        foreach ($customfields as $customfield) {
            if (!empty($_POST[$customfield['fieldid']])) {
                $filtered_var = filter_var(esc_textarea($_POST[$customfield['fieldid']]), FILTER_SANITIZE_STRING);
                update_post_meta($order_id, $customfield['fieldid'], $filtered_var);
            }
        }

        if (!empty($_POST['tax_information'])) {
            $filtered_var = filter_var(esc_textarea($_POST['tax_information']), FILTER_SANITIZE_STRING);
            update_post_meta($order_id, 'tax_information', $filtered_var);
            if ($user_id = get_current_user_id()) {
                update_user_meta($user_id, 'tax_information', $filtered_var);
            }
        }
    }

//    /*
//     * add "my Bookings" to "my Account"
//     */
//
//    function emp_account_menu_items($items)
//    {
//        if (function_exists('em_get_my_bookings_url')) {
//            //add new
//            $part1 = array_slice($items, 0, 1, true);
//            $part2 = array_slice($items, 1, count($items), true);
//            //display order
//            $part_new = [];
//
//            if (get_option('dbem_my_bookings_page'))
//                $part_new['my-bookings'] = __('Bookings', 'events-manager');
//
//            //TODO DSLV in theme!
//            //if role "mitglieder" disable "mitgliedsantrag"
//            $user = wp_get_current_user();
//            if (in_array('mitglieder', (array)$user->roles)) {
//                unset($part_new['mitgliedsantrag']);
//            }
//
//            return array_merge($part1, $part_new, $part2);
//        }
//        return $items;
//    }
//
//    function emp_add_my_account_endpoint()
//    {
//        if (function_exists('em_my_bookings'))
//            add_rewrite_endpoint('my-bookings', EP_PAGES);
//    }
//
//    function emp_bookings_endpoint_content()
//    {
//        if (function_exists('em_my_bookings'))
//            em_my_bookings();
//    }
//
//    function emp_custom_endpoint_title($title)
//    {
//        global $wp_query;
//        $is_endpoint = isset($wp_query->query_vars['my-bookings']);
//        if ($is_endpoint && !is_admin() && is_main_query() && in_the_loop() && is_account_page()) {
//            // New page title.
//            $title = __('My Bookings', 'events-manager');
//            remove_filter('the_title', 'my_custom_endpoint_title');
//        }
//        return $title;
//    }

    /**
     * @throws Exception
     * @deprecated
     */
    public function wc_checkout_check_email($data, $errors)
    {
        // is separate email check necessary? we will check emails on customer save
        $checkout = new WC_Checkout();
        if (is_user_logged_in() || !$checkout->is_registration_required() && empty($data['createaccount'])) {
            return;
        }

        //only on Single-Sign-On
        include_once(ABSPATH . 'wp-admin/includes/plugin.php');
        if (!empty($errors->error_data)
            ||
            (
                !is_plugin_active('v4-oauth/v4oAuth.php')
                &&
                !is_plugin_active('v4-oauth/wp-oauth.php')
            )
            ||
            (
                //don't send this request on every step of Germanized multistep checkout
                isset($_POST['wc_gzdp_step_submit'])
                &&
                (
                    $_POST['wc_gzdp_step_submit'] == 'payment' ||
                    //doesn't exist. check email is send a second time (maybe not bad, validates manipulated input)
                    $_POST['wc_gzdp_step_submit'] == 'submit'
                )
            )
        ) {
            return;
        }

        $email = $data['billing_email'];

        if (!is_user_logged_in() &&
            file_exists(WP_PLUGIN_DIR . '/v4-wp-webhook/includes/v4Webhook.php') && !empty($email)) {
            require_once WP_PLUGIN_DIR . '/v4-wp-webhook/includes/v4Webhook.php';

            if (email_exists($email)) {
                $url = in_array(
                    'v4-oauth/v4oAuth.php',
                    apply_filters('active_plugins', get_option('active_plugins'))
                ) ? '?lost_password=1crm' : '/wp-admin';
                $msg = sprintf(__(
                    "The given e-mail address is already registered with us. If you have forgotten your " .
                    "password, please use the <a href='%s'>Forgot password?</a> function on the login page",
                    "v4-wp-webhook"
                ), $url);
                throw new Exception(__($msg, 'v4-wp1crm-extensions'), 400);
            }
        }
    }

    public function woocommerce_form_field_info($field, $key, $args, $value){
        if(!is_array($args['class'])) $args['class'] = [];
        $args['class'][] = 'form-row';
        return "<p class='" . implode($args['class']) . "'>{$args['label']}</p>";
    }

    public function order_for_customer()
    {
        //only if logged in, and current user has sole
        $user = wp_get_current_user();
        $wp1crm = get_option('wp1crm');
        $roles = $user->roles??[];
        $for_customer_roles = $wp1crm['order_for_customer_roles']??[];
        if (count(array_intersect($roles, $for_customer_roles)) > 0) {
            ?>
            <script>
                (function($){
                    $(document).ready(function(){
                        /*
                            toggle
                         */
                        //init
                        let $container = $('.v4-for-customer');
                        $container.hide();
                        //events
                        $('#v4-is-for-customer').on('change', function(){
                            let checked = this.checked;
                            let $container = $('.v4-for-customer');
                            let $billing = $('.woocommerce-billing-fields');
                            let $shipping = $('.woocommerce-shipping-fields');
                            if(checked){
                                $container.stop().slideDown();
                                $billing.stop().slideUp();
                                $shipping.stop().slideUp();
                            } else {
                                $container.stop().slideUp();
                                $billing.stop().slideDown();
                                $shipping.stop().slideDown();
                            }
                        });
                        /*
                            existing
                         */
                        //init
                        let $existing_msg_field = $('.v4-for-customer-fields__use-existing');
                        $existing_msg_field.hide();
                        //events
                        $('#v4-for-customer-id').on('change', function(){
                            let filled = this.value.length > 0;
                            let $existing_msg_field = $('.v4-for-customer-fields__use-existing');
                            let $fields = $('.v4-for-customer-fields');
                            if(filled){
                                $existing_msg_field.stop().slideDown();
                                $fields.stop().slideUp();
                            } else {
                                $existing_msg_field.stop().slideUp();
                                $fields.stop().slideDown();
                            }
                        });
                        /*
                            copy
                         */
                        $('#v4-for-customer-copy-from-billing').on('click', function(){
                            $('.v4-for-customer-fields--shipping input')
                                .each(function(){
                                    let billing_name = this.name.replace('shipping', 'billing');
                                    $(this).val($('input[name="' + billing_name + '"]').val());
                                });
                            $('.v4-for-customer-fields--shipping select')
                                .each(function(){
                                    let billing_name = this.name.replace('shipping', 'billing');
                                    $(this).val($('select[name="' + billing_name + '"]').val());
                                });
                        });
                    });
                })(jQuery);
            </script>
            <p class="form-row">
                <label>
                    <input type="hidden" value="0" name="is_for_customer">
                    <input type="checkbox" value="1" name="is_for_customer" id="v4-is-for-customer">
                    <span>
                        Ich bestelle für einen meiner Kunden <br>
                        (Der Kunde hat die Datenschutzbestimmungen, die Allgemeinen Geschäftsbedingungen und
                        ggf. die Widerrufsbestimmungen gelesen)
                    </span>
                </label>
            </p>
            <p class="form-row v4-for-customer">
                <label>
                    <span>bestehender Kunde (1CRM Kontakt ID)</span>
                    <input type="text" name="for_customer[id]" id="v4-for-customer-id">
                    <span class="description">
                        Wenn dieses Feld gefüllt ist, werden automatisch die Adressen aus dem CRM verwendet.
                    </span>
                </label>
            </p>
            <?php
        }
    }

    public function order_for_customer_billing()
    {
        $fields = apply_filters('v4/wp1crm/order_for_customer_fields/billing', [
            'billing_salutation' => [
                'type' => 'select',
                'is_meta' => true,
                'title' => 'Anrede',
                'options' => [
                    '' => '- keine -',
                    '1' => 'Herr',
                    '2' => 'Frau',
                    '3' => 'divers',
                ],
                'class' => 'form-row-wide'
            ],
            'first_name' => [
                'title' => 'Vorname',
                'class' => 'form-row-first'
            ],
            'last_name' => [
                'title' => 'Nachname',
                'class' => 'form-row-last'
            ],
            'company' => [
                'title' => 'Firmenname',
                'class' => 'form-row-wide'
            ],
            'address_1' => [
                'title' => 'Straße',
                'class' => 'form-row-wide'
            ],
            'postcode' => [
                'title' => 'Postleitzahl',
                'class' => 'form-row-first'
            ],
            'city' => [
                'title' => 'Ort / Stadt',
                'class' => 'form-row-last'
            ],
            'country' => [
                'type' => 'select',
                'title' => 'Land',
                'options' => WC()->countries->get_countries(),
                'default' => 'DE',
                'class' => 'form-row-wide'
            ],
            'phone' => [
                'title' => 'Telefon',
                'class' => 'form-row-wide'
            ],
            'email' => [
                'type' => 'email',
                'title' => 'E-Mail-Adresse',
                'class' => 'form-row-wide'
            ],
        ]);
        $this->order_for_customer_address($fields);
    }

    public function order_for_customer_shipping()
    {
        $fields = apply_filters('v4/wp1crm/order_for_customer_fields/shipping', [
            'shipping_salutation' => [
                'type' => 'select',
                'is_meta' => true,
                'title' => 'Anrede',
                'options' => [
                    '' => '- keine -',
                    '1' => 'Herr',
                    '2' => 'Frau',
                    '3' => 'divers',
                ],
                'class' => 'form-row-wide'
            ],
            'first_name' => [
                'title' => 'Vorname',
                'class' => 'form-row-first'
            ],
            'last_name' => [
                'title' => 'Nachname',
                'class' => 'form-row-last'
            ],
            'company' => [
                'title' => 'Firmenname',
                'class' => 'form-row-wide'
            ],
            'address_1' => [
                'title' => 'Straße',
                'class' => 'form-row-wide',
            ],
            'postcode' => [
                'title' => 'Postleitzahl',
                'class' => 'form-row-first'
            ],
            'city' => [
                'title' => 'Ort / Stadt',
                'class' => 'form-row-last'
            ],
            'country' => [
                'type' => 'select',
                'title' => 'Land',
                'options' => WC()->countries->get_countries(),
                'default' => 'DE',
                'class' => 'form-row-wide'
            ],
        ]);
        $this->order_for_customer_address($fields, 'shipping');
    }

    public function order_for_customer_address($fields, $type = 'billing')
    {
        //only if logged in, and current user has sole
        $user = wp_get_current_user();
        $wp1crm = get_option('wp1crm');
        $roles = $user->roles??[];
        $for_customer_roles = $wp1crm['order_for_customer_roles']??[];
        if (count(array_intersect($roles, $for_customer_roles)) > 0) {
            ?>
            <div class="v4-for-customer">
                <h3><?php echo $type === 'billing' ? 'Rechnungsdetails' : 'Lieferdetails' ?> des Kunden</h3>
                <div class="v4-for-customer-fields v4-for-customer-fields--<?= $type ?>">
                    <?php
                    if ($type === 'shipping') {
                        ?>
                        <p class="form-row">
                            <label>
                                <button id="v4-for-customer-copy-from-billing"
                                        onclick="return false;">
                                    aus den Rechnungsdetails kopieren
                                </button>
                            </label>
                        </p>
                        <?php
                    }
                    foreach ($fields as $k => $field) {
                        //set field name
                        if ($field['is_meta'] ?? false) {
                            $field_name = "for_customer[meta_data][$k][value]";
                            $hidden_field_name = "for_customer[meta_data][$k][key]";
                            ?>
                            <input type="hidden" name="<?= $hidden_field_name ?>" value="<?= $k ?>">
                            <?php
                        } else {
                            $field_name = "for_customer[$type][$k]";
                        }

                        if (isset($field['type']) && $field['type'] === 'select') {
                            ?>
                            <p class="form-row <?= $field['class'] ?? '' ?>">
                                <label>
                                    <span><?= $field['title'] ?></span>
                                    <select name="<?= $field_name ?>">
                                        <?php
                                        foreach ($field['options'] as $v => $option) {
                                            $selected = '';
                                            if (isset($field['default']) && $field['default'] === $v) {
                                                $selected = 'selected';
                                            }
                                            ?>
                                            <option value="<?= $v ?>" <?= $selected ?>><?= $option ?></option>
                                            <?php
                                        }
                                        ?>
                                    </select>
                                </label>
                            </p>
                            <?php
                        } else {
                            ?>
                            <p class="form-row <?= $field['class'] ?? '' ?>">
                                <label>
                                    <span><?= $field['title'] ?></span>
                                    <input type="<?= $field['type'] ?? 'text' ?>"
                                           name="<?= $field_name ?>">
                                </label>
                            </p>
                            <?php
                        }
                    }
                    ?>
                </div>
                <div class="v4-for-customer-fields__use-existing">
                    <p class="form-row">Es werden die Daten des bestehenden Kontaktes verwendet</p>
                </div>
            </div>
            <?php
        }
    }

    public function order_for_customer_field_update($order_id)
    {
        //Iterate and create Formfield
        if (!empty($_POST['for_customer'])) {
            $for_customer = $_POST['for_customer'];

            //id given?
            if (!empty($for_customer['id'])) {
                $for_customer = ['id' => $for_customer['id']];
            } else {
                //set base data from billing
                $for_customer['email'] = $for_customer['billing']['email'];
                $for_customer['first_name'] = $for_customer['billing']['first_name'];
                $for_customer['last_name'] = $for_customer['billing']['last_name'];
                $for_customer['role'] = 'customer';
            }

            update_post_meta($order_id, 'for_customer', $for_customer);
        }
    }
}
